# MiloDB by FrozenWolf is marked with CC0 1.0.
# To view a copy of this license, visit http://creativecommons.org/publicdomain/zero/1.0
from abc import ABC, abstractmethod
from milodb_client.output.format.i_formatter import IFormatter
from milodb_common.output.print.i_printer import IPrinter

class IFormatterCreator(ABC):
    @abstractmethod
    def get_name(self) -> str:
        pass

    @abstractmethod
    def create(self, printer: IPrinter) -> IFormatter:
        pass

class IFormatterFactory(ABC):
    @abstractmethod
    def get_list_of_formatter_names(self) -> list[str]:
        pass

    @property
    @abstractmethod
    def map_of_name_to_formatter_creator(self) -> dict[str, IFormatterCreator]:
        pass
